#include <Wire.h>
#define LPSADDR 0x5C

uint32_t pressure = 0 ;
float temperature = 0 ;

uint8_t i2cBuff[6] ;

// Register読み取り
uint8_t readLPS(uint8_t addr) {
  uint8_t readData ;
  // Read Address Set
  Wire.beginTransmission(LPSADDR);
  Wire.write(addr);
  Wire.endTransmission();
  
  // 1Byte Read
  Wire.requestFrom(LPSADDR, 1);
  while(Wire.available()<1);
  readData = Wire.read();

  return readData ;
}

// Register読み取り
uint8_t readLPSBuff(uint8_t addr,int readLen) {
  uint8_t readData ;
  // Read Address Set
  Wire.beginTransmission(LPSADDR);
  Wire.write(addr|0x80);
  Wire.endTransmission();
  
  // 1Byte Read
  Wire.requestFrom(LPSADDR , readLen);
  while(Wire.available() < readLen);
  for (int i=0;i<readLen;i++) {
    i2cBuff[i] = Wire.read();
  }

  Serial.print("i2cBuff=") ;
  for (int i=0;i<readLen;i++) {
    Serial.print(i2cBuff[i],HEX) ;
    Serial.print(" ") ;
  }
 Serial.println() ;
 
  return readData ;
}

// 気圧センサー読み取り
bool readLPS_Sencer( ) {
  uint8_t stat = readLPS(0x27);
  uint8_t fifoStat = readLPS(0x2F);
  readLPSBuff(0x28,3) ;
  uint8_t pxl = i2cBuff[0] ;
  uint8_t pl = i2cBuff[1] ;
  uint8_t ph = i2cBuff[2] ;
  readLPSBuff(0x2B,2) ;
  uint8_t tl = i2cBuff[0] ;
  uint8_t th = i2cBuff[1] ;

//  uint8_t pxl = readLPS(0x28) ;
//  uint8_t pl = readLPS(0x29) ;
//  uint8_t ph = readLPS(0x2A) ;
//  uint8_t tl = readLPS(0x2B) ;
//  uint8_t th = readLPS(0x2C) ;
//  uint8_t fifoStat = readLPS(0x2F);

  // combine bytes
  pressure = (int32_t)(int8_t)ph << 16 | (uint16_t)pl << 8 | pxl;
  pressure /= 4096 ;
  int16_t temp = (int16_t)th << 8 | tl ;
  temperature = 42.5 + (float)temp / 480;

  Serial.print("stat = ") ;
  Serial.print(stat,HEX) ;
  Serial.print("  fifoStat = ") ;
  Serial.print(fifoStat,HEX) ;
  Serial.print("  ") ;
  
  return ((stat & 0x03) == 0x03) && ((fifoStat & 0xC0) == 0xC0)  ;
}

// 1バイトコマンドを送信
void sendLPS(uint8_t addr , uint8_t command) {
  Wire.beginTransmission(LPSADDR);
  Wire.write(addr);
  Wire.write(command);
  Wire.endTransmission();
}

void setup() {
  Wire.begin();

  Serial.begin(9600);
  while (!Serial); // Leonardo: wait for serial monitor
  Serial.println("\nLPS25HB 気圧センサーテスト - FIFO有効/分割読み出し");

  // -- Read WHO AM I
  uint8_t deviceID = readLPS(0x0F) ;
  Serial.print("Device ID = ") ;
  Serial.println(deviceID,HEX) ;

  // --- Reset LPS25HB
  sendLPS(0x21,0x04);   // ソフトウェアリセット
  while(readLPS(0x21)); // リセット確認
  delay(100);
  sendLPS(0x21,0x80);   // リブートメモリ
  while(readLPS(0x21)); // リブート確認
  delay(100);
  sendLPS(0x20,0xc0);   // アクティブモード、アウトプット25Hz
                        // ※VCCに5V入れた場合、1Hzだと何故か誤差が大きい
                        // ※この設定だと+2hPa +1℃位のズレで収まる
  delay(100);
  sendLPS(0x21,0x40);   // FiFO有効
  sendLPS(0x2E,0xC7);   // FIFO平均モード 8サンプル
                        // ※サンプル数が大きすぎるとFIFOがFULLになるまでに時間がかかる
  delay(1000);
}

void loop() {
  // ----- Read 気圧センサー
  bool ret = readLPS_Sencer( ) ;
  Serial.print("ret = ") ;
  Serial.print(ret) ;
  Serial.print("  /  pressure = ") ;
  Serial.print(pressure) ;
  Serial.print("  /  Temperature = ") ;
  Serial.println(temperature) ;

  delay(10*1000) ; // 10秒ディレイ
}
